@@ -17,7 +17,7 @@ use App::ClusterSSH::Cluster;
 
 my $clusters;
 my %old_clusters;
-my @app_specific = (qw/ command title comms method /);
+my @app_specific = (qw/ command title comms method parent /);
 
 # list of config items to not write out when writing the default config
 my @ignore_default_config = (qw/ user /);
@@ -37,11 +37,15 @@ my %default_config = (
     key_addhost             => "Control-Shift-plus",
     key_clientname          => "Alt-n",
     key_history             => "Alt-h",
+    key_localname           => "Alt-l",
     key_retilehosts         => "Alt-r",
+    key_macros_enable       => "Alt-p",
     key_paste               => "Control-v",
+    key_username            => "Alt-u",
     mouse_paste             => "Button-2",
     auto_quit               => "yes",
     auto_close              => 5,
+    use_natural_sort        => 0,
     window_tiling           => "yes",
     window_tiling_direction => "right",
     console_position        => "",
@@ -67,6 +71,8 @@ my %default_config = (
     telnet_args  => "",
     ssh          => 'ssh',
     ssh_args     => "",
+    sftp         => 'sftp',
+    sftp_args    => "",
 
     extra_cluster_file       => '',
     external_cluster_command => '',
@@ -80,13 +86,20 @@ my %default_config = (
     command             => q{},
     max_host_menu_items => 30,
 
+    macros_enabled   => 'yes',
+    macro_servername => '%s',
+    macro_hostname   => '%h',
+    macro_username   => '%u',
+    macro_newline    => '%n',
+    macro_version    => '%v',
+
     max_addhost_menu_cluster_items => 6,
     menu_send_autotearoff          => 0,
     menu_host_autotearoff          => 0,
 
     use_all_a_records => 0,
 
-    send_menu_xml_file => $ENV{HOME} . '/.csshrc_send_menu',
+    send_menu_xml_file => $ENV{HOME} . '/.clusterssh/send_menu',
 
     # don't set username here as takes precendence over ssh config
     user => '',
@@ -102,9 +115,10 @@ sub new {
     $comms = 'telnet'  if ( $comms eq 'tel' );
     $comms = 'console' if ( $comms eq 'con' );
     $comms = 'ssh'     if ( $comms eq 'lusterssh' );
+    $comms = 'sftp'    if ( $comms eq 'sftp' );
 
     # list of allowed comms methods
-    if ( 'ssh rsh telnet console' !~ m/\b$comms\b/ ) {
+    if ( 'ssh rsh telnet sftp console' !~ m/\b$comms\b/ ) {
         $self->{comms} = 'ssh';
     }
     else {
@@ -169,12 +183,10 @@ sub validate_args {
         );
     }
 
-    #    # Don't search for the path to the binary - assume it is on the path
-    #    # or defined correctly in the config.
-    #    if( !-e $self->{ $self->{comms} } )
-    #    {
-    #        $self->{ $self->{comms} } = $self->find_binary( $self->{comms} );
-    #    }
+    # check the terminal has been found correctly
+    if ( !-e $self->{terminal} ) {
+        $self->{terminal} = $self->find_binary( $self->{terminal} );
+    }
 
     return $self;
 }
@@ -377,6 +389,25 @@ sub write_user_config_file {
     return $self;
 }
 
+# search given directories for the given file
+sub search_dirs {
+    my ( $self, $file, @directories ) = @_;
+
+    my $path;
+
+    foreach my $dir (@directories) {
+        $self->debug( 3, "Looking for $file in $dir" );
+
+        if ( -f $dir . '/' . $file && -x $dir . '/' . $file ) {
+            $path = $dir . '/' . $file;
+            $self->debug( 2, "Found at $path" );
+            last;
+        }
+    }
+
+    return $path;
+}
+
 # could use File::Which for some of this but we also search a few other places
 # just in case $PATH isnt set up right
 sub find_binary {
@@ -395,39 +426,39 @@ sub find_binary {
     # if not found, strip the path and look again
     if ( $binary =~ m!^/! ) {
         if ( -f $binary ) {
-            $self->debug( 2, "$binary already fully qualified" );
+            $self->debug( 2, "Already have full path to in $binary" );
             return $binary;
         }
         else {
-            $self->debug( 2, "$binary not found - re-searching" );
+            $self->debug( 2, "Full path for $binary incorrect; searching" );
             $binary =~ s!^.*/!!;
         }
     }
 
     my $path;
     if ( !-x $binary || substr( $binary, 0, 1 ) ne '/' ) {
+        $path = $self->search_dirs( $binary, split( /:/, $ENV{PATH} ) );
 
-        foreach (
-            split( /:/, $ENV{PATH} ), qw!
-            /bin
-            /sbin
-            /usr/sbin
-            /usr/bin
-            /usr/local/bin
-            /usr/local/sbin
-            /opt/local/bin
-            /opt/local/sbin
-            !
-            )
-        {
-            $self->debug( 3, "Looking in $_" );
-
-            if ( -f $_ . '/' . $binary && -x $_ . '/' . $binary ) {
-                $path = $_ . '/' . $binary;
-                $self->debug( 2, "Found at $path" );
-                last;
-            }
+        # if it is on $PATH then no need to qualitfy the path to it
+        # keep it as it is
+        if ($path) {
+            return $binary;
         }
+        else {
+            $path = $self->search_dirs(
+                $binary, qw!
+                    /bin
+                    /sbin
+                    /usr/sbin
+                    /usr/bin
+                    /usr/local/bin
+                    /usr/local/sbin
+                    /opt/local/bin
+                    /opt/local/sbin
+                    !
+            );
+        }
+
     }
     else {
         $self->debug( 2, "Already configured OK" );
@@ -507,6 +538,10 @@ Read in configuration from given filename
 
 Validate and apply all configuration loaded at this point
 
+=item $path = $config->search_dirs('<name>', @seaarch_directories);
+
+Search the given directories for the name given.  Return undef if not found.
+
 =item $path = $config->find_binary('<name>');
 
 Locate the binary <name> and return the full path.  Doesn't just search 
@@ -534,7 +569,7 @@ Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 1999-2010 Duncan Ferguson.
+Copyright 1999-2015 Duncan Ferguson.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
